<?php

/**
 * Test: Object event handlers.
 *
 * @author     David Grudl
 * @package    Nette
 */



require dirname(__FILE__) . '/../bootstrap.php';



class TestClass extends Object
{
	public $onPublic;

	static public $onPublicStatic;

	protected $onProtected;

	private $onPrivate;

}



function handler($obj)
{
	$obj->counter++;
}



class Handler
{
	function __invoke($obj)
	{
		$obj->counter++;
	}
}



$obj = new TestClass;
$obj->onPublic();

$var = new stdClass;

$obj->onPublic[] = 'handler';

$obj->onPublic($var);
Assert::same( 1, $var->counter );



$obj->onPublic[] = new Handler;

$obj->onPublic($var);
Assert::same( 3, $var->counter );



Assert::exception(create_function('', 'extract($GLOBALS[0]['.array_push($GLOBALS[0], array('obj'=>$obj)).'-1], EXTR_REFS);
	$obj->onPublicStatic(123);
'), 'MemberAccessException', 'Call to undefined method TestClass::onPublicStatic().');


Assert::exception(create_function('', 'extract($GLOBALS[0]['.array_push($GLOBALS[0], array('obj'=>$obj)).'-1], EXTR_REFS);
	$obj->onProtected(123);
'), 'MemberAccessException', 'Call to undefined method TestClass::onProtected().');


Assert::exception(create_function('', 'extract($GLOBALS[0]['.array_push($GLOBALS[0], array('obj'=>$obj)).'-1], EXTR_REFS);
	$obj->onPrivate(123);
'), 'MemberAccessException', 'Call to undefined method TestClass::onPrivate().');


Assert::exception(create_function('', 'extract($GLOBALS[0]['.array_push($GLOBALS[0], array('obj'=>$obj)).'-1], EXTR_REFS);
	$obj->onUndefined(123);
'), 'MemberAccessException', 'Call to undefined method TestClass::onUndefined().');

Assert::exception(create_function('', 'extract($GLOBALS[0]['.array_push($GLOBALS[0], array('obj'=>$obj)).'-1], EXTR_REFS);
	$obj->onPublic = \'string\';
	$obj->onPublic();
'), 'UnexpectedValueException', 'Property TestClass::$onPublic must be array or NULL, string given.');
